//----------------------------------------------------------------------------
//      _____
//     *     *
//    *____   *____
//   * *===*   *==*
//  *___*===*___**  AVNET
//       *======*
//        *====*
//----------------------------------------------------------------------------
//
// This design is the property of Avnet.  Publication of this
// design is not authorized without written consent from Avnet.
//
// Please direct any questions to:  technical.support@avnet.com
//
// Disclaimer:
//    Avnet, Inc. makes no warranty for the use of this code or design.
//    This code is provided  "As Is". Avnet, Inc assumes no responsibility for
//    any errors, which may appear in this code, nor does it make a commitment
//    to update the information contained herein. Avnet, Inc specifically
//    disclaims any implied warranties of fitness for a particular purpose.
//                     Copyright(c) 2013 Avnet, Inc.
//                             All rights reserved.
//
//----------------------------------------------------------------------------
//
// Create Date:         May 14, 2013
// Module Name:         zynq_io.h
// Project Name:        Avnet Retarget Example
//
// Tool versions:       ARM DS-5 r14
//
// Description:         Text-based example application console for
//                      ZedBoard Getting Started Design with DS-5
//
// Dependencies:
//
// Revision:            May 14, 2013: 1.00 Initial version
//                      Aug 14, 2013: 1.01 Updated for customer example
//
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// System Includes.

//----------------------------------------------------------------------------
// Local Includes.

#include "zynq_io.h"
#include "zynq_types.h"

//----------------------------------------------------------------------------
// Function Definitions.


//----------------------------------------------------------------------------
// Performs an input operation for an 8-bit memory location through a
// synchronized read from memory.
//
//     Arguments:
//
//         address
//             Target address for performing the memory read operation.
//
//     Return:
//         The value read from the specified input address.
//----------------------------------------------------------------------------
int8u zynq_read8(int32u address)
{
	/* read the contents of the I/O location and then synchronize the I/O
	 * such that the I/O operation completes before proceeding on
	 */
	dmb();
	return *(volatile int8u *) address;
}

//----------------------------------------------------------------------------
// Performs an input operation for an 16-bit memory location through a
// synchronized read from memory.
//
//     Arguments:
//
//         address
//             Target address for performing the memory read operation.
//
//     Return:
//         The value read from the specified input address.
//----------------------------------------------------------------------------
int16u zynq_read16(int32u address)
{
	/* read the contents of the I/O location and then synchronize the I/O
	 * such that the I/O operation completes before proceeding on
	 */
	dmb();
	return *(volatile int16u *) address;
}

//----------------------------------------------------------------------------
// Performs an input operation for an 32-bit memory location through a
// synchronized read from memory.
//
//     Arguments:
//
//         address
//             Target address for performing the memory read operation.
//
//     Return:
//         The value read from the specified input address.
//----------------------------------------------------------------------------
int32u zynq_read32(int32u address)
{
	/* read the contents of the I/O location and then synchronize the I/O
	 * such that the I/O operation completes before proceeding on
	 */
	dmb();
	return *(volatile int32u *) address;
}

//----------------------------------------------------------------------------
// Performs an output operation for an 8-bit memory location through a
// synchronized write to memory.
//
//     Arguments:
//
//         address
//             Target address for performing the memory write operation.
//
//         value
//             Value that is written to the target memory location.
//
//     Return:
//         None.
//----------------------------------------------------------------------------
void zynq_write8(int32u address, int8u value)
{
	/* write the contents of the I/O location and then synchronize the I/O
	 * such that the I/O operation completes before proceeding on
	 */
	*(volatile int8u *) address = value;
	dmb();
}

//----------------------------------------------------------------------------
// Performs an output operation for an 16-bit memory location through a
// synchronized write to memory.
//
//     Arguments:
//
//         address
//             Target address for performing the memory write operation.
//
//         value
//             Value that is written to the target memory location.
//
//     Return:
//         None.
//----------------------------------------------------------------------------
void zynq_write16(int32u address, int16u value)
{
	/* write the contents of the I/O location and then synchronize the I/O
	 * such that the I/O operation completes before proceeding on
	 */
	*(volatile int16u *) address = value;
	dmb();
}

//----------------------------------------------------------------------------
// Performs an output operation for an 32-bit memory location through a
// synchronized write to memory.
//
//     Arguments:
//
//         address
//             Target address for performing the memory write operation.
//
//         value
//             Value that is written to the target memory location.
//
//     Return:
//         None.
//----------------------------------------------------------------------------
void zynq_write32(int32u address, int32u value)
{
	/* write the contents of the I/O location and then synchronize the I/O
	 * such that the I/O operation completes before proceeding on
	 */
	*(volatile int32u *) address = value;
	dmb();
}
