//----------------------------------------------------------------------------
//      _____
//     *     *
//    *____   *____
//   * *===*   *==*
//  *___*===*___**  AVNET
//       *======*
//        *====*
//----------------------------------------------------------------------------
//
// This design is the property of Avnet.  Publication of this
// design is not authorized without written consent from Avnet.
//
// Please direct any questions to:  technical.support@avnet.com
//
// Disclaimer:
//    Avnet, Inc. makes no warranty for the use of this code or design.
//    This code is provided  "As Is". Avnet, Inc assumes no responsibility for
//    any errors, which may appear in this code, nor does it make a commitment
//    to update the information contained herein. Avnet, Inc specifically
//    disclaims any implied warranties of fitness for a particular purpose.
//                     Copyright(c) 2013 Avnet, Inc.
//                             All rights reserved.
//
//----------------------------------------------------------------------------
//
// Create Date:         May 14, 2013
// Module Name:         zynq_ps_uart.h
// Project Name:        Avnet Retarget Example
//
// Tool versions:       ARM DS-5 r14
//
// Description:         Text-based example application console for
//                      ZedBoard Getting Started Design with DS-5
//
// Dependencies:
//
// Revision:            May 14, 2013: 1.00 Initial version
//                      Aug 14, 2013: 1.01 Updated for customer example
//
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// Prevent circular inclusions by using preprocessor protection macros.

#ifndef ZYNQ_PS_UART_H
#define ZYNQ_PS_UART_H

#ifdef __cplusplus
extern "C" {
#endif

//----------------------------------------------------------------------------
// Local Includes.

#include "zynq_types.h"
#include "zynq_io.h"

//----------------------------------------------------------------------------
// Constant Definitions.

/* Maximum % error rate allowed during calculations is 3. */
#define PS_UART_MAX_BAUD_ERROR_RATE	   3

/* Define the target PS UART baud rate here. */
#define PS_UART_BAUD_RATE              115200

/* Define the target PS UART device used for STDIN and STDOUT here. */
#define STDIN_BASEADDRESS		       ZYNQ_PS_UART_1_BASEADDR
#define STDOUT_BASEADDRESS		       ZYNQ_PS_UART_1_BASEADDR

#define ZYNQ_PS_UART_0_BASEADDR        0xE0000000
#define ZYNQ_PS_UART_0_CLOCK_HZ	       50000000
#define ZYNQ_PS_UART_0_INTR		       51

#define ZYNQ_PS_UART_1_BASEADDR        0xE0001000
#define ZYNQ_PS_UART_1_CLOCK_HZ	       50000000
#define ZYNQ_PS_UART_1_INTR		       75

//----------------------------------------------------------------------------
// PS UART register offsets.
#define PS_UART_CONTROL_OFFSET		   0x00
#define PS_UART_MODE_OFFSET		       0x04
#define PS_UART_INT_ENABLE_OFFSET      0x08
#define PS_UART_INT_DISABLE_OFFSET     0x0C
#define PS_UART_INT_MASK_OFFSET        0x10
#define PS_UART_INT_STATUS_OFFSET      0x14
#define PS_UART_BAUD_GEN_OFFSET	       0x18
#define PS_UART_RX_TIMEOUT_OFFSET      0x1C
#define PS_UART_RX_FIFO_TRIGGER_OFFSET 0x20
#define PS_UART_MODEM_CONTROL_OFFSET   0x24
#define PS_UART_MODEM_STAUTS_OFFSET    0x28
#define PS_UART_CHANNEL_STATUS_OFFSET  0x2C
#define PS_UART_RX_TX_FIFO_OFFSET      0x30
#define PS_UART_BAUD_DIV_OFFSET	       0x34
#define PS_UART_FLOW_DELAY_OFFSET      0x38
#define PS_UART_TX_FIFO_TRIGGER_OFFSET 0x44

/** @name Channel Status Register
 *
 * The channel status register (CSR) is provided to enable the control logic
 * to monitor the status of bits in the channel interrupt status register,
 * even if these are masked out by the interrupt mask register.
 *
 * @{
 */

#define PS_UART_CHANNEL_STATUS_FLOWDEL 0x00001000	 /**< RX FIFO fill over flow delay */
#define PS_UART_CHANNEL_STATUS_TACTIVE 0x00000800
#define PS_UART_CHANNEL_STATUS_RACTIVE 0x00000400
#define PS_UART_CHANNEL_STATUS_DMS     0x00000200
#define PS_UART_CHANNEL_STATUS_TOUT    0x00000100
#define PS_UART_CHANNEL_STATUS_PARITY  0x00000080
#define PS_UART_CHANNEL_STATUS_FRAME   0x00000040
#define PS_UART_CHANNEL_STATUS_OVER    0x00000020
#define PS_UART_CHANNEL_STATUS_TXFULL  0x00000010
#define PS_UART_CHANNEL_STATUS_TXEMPTY 0x00000008
#define PS_UART_CHANNEL_STATUS_RXFULL  0x00000004
#define PS_UART_CHANNEL_STATUS_RXEMPTY 0x00000002
#define PS_UART_CHANNEL_STATUS         0x00000001

//----------------------------------------------------------------------------
// Function Prototypes.

int8u zynq_uart_get_char(void);
void zynq_uart_initialize(void);
void zynq_uart_put_char(char character);

//----------------------------------------------------------------------------
#ifdef __cplusplus
}
#endif

#endif // ZYNQ_IO_H
